function cleanMDT_nTens(inputPath,outputPath,genericName)
%cleanMDT_nTens      Remove small tensors for DBF solution and scale them for visualization.
%
%   cleanMDT_nTens(inputPath,outputPath,genericName) 
%   The function looks for a DBF solution in directory INPUTPATH and with generic name GENERICNAME, then
%   writes new files in OUTPUTPATH where small tensors (non plausible ones) where removed from the multi tensor DBF
%   solution and a version of files with tensor scales according to the size compartment for visualization
%   in DTITK.
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$


percentageBiggest = 0.2;

%open files created before
nii = nifti([ inputPath 'tensors1_' genericName '.nii']);
t1 = nii2matlab(nii.dat(:,:,:,:,:));

nii = nifti([ inputPath 'tensors2_' genericName '.nii']);
t2 = nii2matlab(nii.dat(:,:,:,:,:));

nii = nifti([ inputPath 'tensors3_' genericName '.nii']);
t3 = nii2matlab(nii.dat(:,:,:,:,:));


nii = nifti([ inputPath 'nTens_' genericName '.nii']);
nTens = nii2matlab(nii.dat(:,:,:,:,:));

nii = nifti([ inputPath 'sizeCompartment_' genericName '.nii']);
amount = nii2matlab(nii.dat(:,:,:,:,:));

mask = nTens > 0 ; %only for this positions

t1s = t1;
t2s = t2;
t3s = t3;

[ny,nx,nz] = size(mask);

for pz=1:nz; for px=1:nx;  for py=1:ny
            
    if mask(py,px,pz)==0
        continue;
    end
    
    for i=nTens(py,px,pz):-1:2 % run backward ie from smallers to biggers
        if amount(py,px,pz,i) <= amount(py,px,pz,1)*percentageBiggest
            nTens(py,px,pz) = nTens(py,px,pz) - 1;
            switch i
                case 2
                    t2(py,px,pz,1,:) = zeros(6,1);
                case 3
                    t3(py,px,pz,1,:) = zeros(6,1);
            end
            amount(py,px,pz,i) = 0;
        end
        
    end
    
    amount(py,px,pz,:) = amount(py,px,pz,:) / sum(amount(py,px,pz,:)); %re-normalize
    
    %scale only for DTITK vizualization
    t1s(py,px,pz,1,:) =  t1(py,px,pz,1,:) * amount(py,px,pz,1);
    t2s(py,px,pz,1,:) =  t2(py,px,pz,1,:) * amount(py,px,pz,2);
    t3s(py,px,pz,1,:) =  t3(py,px,pz,1,:) * amount(py,px,pz,3);
    

end; end; end


saveMatlab2Nifti([ outputPath 'tensors1_cleaned_' genericName '.nii'],t1,'MATRIX_SYM',2,2,2);
saveMatlab2Nifti([ outputPath 'tensors2_cleaned_' genericName '.nii'],t2,'MATRIX_SYM',2,2,2);
saveMatlab2Nifti([ outputPath 'tensors3_cleaned_' genericName '.nii'],t3,'MATRIX_SYM',2,2,2);

saveMatlab2Nifti([ outputPath 'tensors1_cleaned_scaled_' genericName '.nii'],t1s,'MATRIX_SYM',2,2,2);
saveMatlab2Nifti([ outputPath 'tensors2_cleaned_scaled_' genericName '.nii'],t2s,'MATRIX_SYM',2,2,2);
saveMatlab2Nifti([ outputPath 'tensors3_cleaned_scaled_' genericName '.nii'],t3s,'MATRIX_SYM',2,2,2);


saveMatlab2Nifti([ outputPath 'sizeCompartmen_cleaned_' genericName '.nii'],amount,'NONE',2,2,2);
saveMatlab2Nifti([ outputPath 'nTens_cleaned_' genericName '.nii'],nTens,'NONE',2,2,2);

fprintf('\n cleaned done\n')


