function [fNDiff,fPDD,fAmountDiff,fSizeClusters] = computeContiniousField(alphasV,BasisDir,basisDNeigh,threshold,mask)
%computeContiniousField      Compute the volume on continuous solutions for discrete DBF solution
%
%   [fNDiff,fPDD,fAmountDiff,fSizeClusters] = computeContiniousField(alphasV,BasisDir,basisDNeigh,threshold,mask,imsRGB) 
%   Given the volume ALPHASV of discrete DBF solutions in discrete directions BASISDIR and orientation-neighborhood structure BASISDNEIGH,
%   the function computes the continuous solution depicted by number FNDIFF of diffusion directions, the diffusion directions FPDD, 
%   the amount of each one FAMOUNTDIFF and the number of fusioned discrete solutions FSIZECLUSTER.
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$


nMaxDifs = 3; %always 3

[ny,nx,nz,nv] = size(alphasV);

fNDiff=zeros(ny,nx,nz);
fPDD=zeros(ny,nx,nz,nMaxDifs,3);
fAmountDiff=zeros(ny,nx,nz,nMaxDifs);
fSizeClusters=zeros(ny,nx,nz,nMaxDifs);


fprintf('\nCompute DBF continuous solution..\n');
for pz=1:nz
    fprintf('\nslice %d',pz);    
    for py=1:ny
        for px=1:nx
            if mask(py,px,pz)==0
                continue;
            end
            alfas = squeeze(alphasV(py,px,pz,:));
            [nCentroids,centroids,amountDiffs,sizeClusters] = giveContinousPDDS(alfas,BasisDir,basisDNeigh,threshold,nMaxDifs);
            amountDiffs = amountDiffs / sum(amountDiffs);
            
            fNDiff(py,px,pz)=nCentroids;
            fPDD(py,px,pz,1:nCentroids,:) = centroids(1:nCentroids,:);
            fAmountDiff(py,px,pz,1:nCentroids) = amountDiffs(1:nCentroids);
            fSizeClusters(py,px,pz,1:nCentroids) = sizeClusters(1:nCentroids);
        end
    end 
end



