function [GA, HRTC, MD, gentr, V] = computeHighRankTensor(Si,S0r,NLEC,nxsyszs,mu,mat,fac_m_i_b_mu,nCoef,T)
%computeHighRankTensor      fit high rank tensor and scalar values [MD, Generalized Anisotrophy (GA)] to DW signals.
%
%   [GA, HRTC, MD, gentr, V] = computeHighRankTensor(Si,S0r,NLEC,nxsyszs,mu,mat,fac_m_i_b_mu,nCoef,T) 
%                       Given the #NLEC DW signals S, and S0r, fit a a high rank tensor as explained in
%                       'Özarslan E. etal, (2003) Generalized diffusion tensor imaging 
%                       and analytical relationships between diffusion tensor imaging 
%                       and high angular resolution diffusion imaging. Magn. Reson. Med.'
%                       The 6 High rank tensor coefficients are given in HRTC with MD and GA.
%                       Use preComputeHRTenData for compute nxsyszs,mu,mat,fac_m_i_b_mu,nCoef,T parameters.
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$

%% FIT B vector
vecB = zeros(nCoef,1);
for m=1:nCoef
    sumi = 0;
    for i=1:NLEC
        if Si(i) == 0 % prevent log(0)
            Si(i) = 1e-8;
        end        
        sumi = sumi + (log(Si(i))-log(S0r))*fac_m_i_b_mu(m,i);
    end
    vecB(m) = - sumi;
end


HRTC = mat \ vecB;


if nCoef == 28
    %compute mean diffusity , rank 6
    MD = (1/7) * (HRTC(1)+HRTC(2)+HRTC(3) + 3 * (HRTC(10)+HRTC(11)+HRTC(12)+HRTC(13)+HRTC(14)+HRTC(15)) + 6*HRTC(28) );
elseif nCoef == 15
    %compute mean diffusity , rank 4
    MD = (1/5) * (HRTC(1)+HRTC(2)+HRTC(3) + 2 * (HRTC(10)+HRTC(11)+HRTC(12)));
else %if  nCoef == 6
    % %compute mean diffusity , rank 2
    MD = (1/3) * (HRTC(1)+HRTC(2)+HRTC(3));
end

%compute gentr
sumk1k2=0;
for k1=1:nCoef
    for k2=1:nCoef
        vGammaA = sort([nxsyszs(k1,1)+nxsyszs(k2,1) nxsyszs(k1,2)+nxsyszs(k2,2) nxsyszs(k1,3)+nxsyszs(k2,3)],2);
        vGamma = [vGammaA(3) vGammaA(2) vGammaA(1)];
        gamma = computeGammaHRT(vGamma,T);
        if gamma ~= 0
            sumk1k2 = sumk1k2 + mu(k1)*mu(k2)*HRTC(k1)*HRTC(k2)*(1/gamma);
        end
    end
end
gentr = (1/(3*MD^2))*sumk1k2;

V = (1/3)*(gentr-(1/3));

epsilon = 1 + 1/(1+5000*V);
%the generalized anisotropy (it is not FA)
GA = 1 - 1 / (1 + (250*V)^epsilon);
