function [centroid,amountDiff] = continuosCollapsing(sizeCluster,idxsCluster,BasisDir,alfas)
%continuosCollapsing      Computes the centroid and amout of a discrete orientation-cluster.
%
%   [centroid,amountDiff] = continuosCollapsing(sizeCluster,idxsCluster,BasisDir,alfas) 
%   where SIZECLUSTER is the cluster's cardinality, IDXCLUSTER indicates the index of each
%   orientation in the membership ALFAS vector, and BASISDIR gives the orientation of each
%   membership. Returns the 3D vector CENTROID , and the amount of diffusion in AMOUNTDIFF.
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$

set = zeros(sizeCluster,3);

%change the sign
set(1,:) = BasisDir(idxsCluster(1),:) * alfas(idxsCluster(1));
for i=2:sizeCluster
    if BasisDir(idxsCluster(i),:) * set(1,:)' < (-BasisDir(idxsCluster(i),:)) * set(1,:)'
        set(i,:) = - BasisDir(idxsCluster(i),:)*alfas(idxsCluster(i)); %weighted sum
    else
        set(i,:) =   BasisDir(idxsCluster(i),:)*alfas(idxsCluster(i));        
    end
end

centroid = sum(set,1);
centroid = centroid / norm(centroid); %normalize to unity norm


amountDiff = 0;
for i=1:sizeCluster
    %the projection over the centroid of each component direction
    amountDiff = amountDiff + abs(centroid*BasisDir(idxsCluster(i),:)') * alfas(idxsCluster(i)) ;
end

    
