function continuous2NIFTI(fNDiff,fPDD,fAmountDiff,mask,path,name,DPDD,DTra,spx,spy,spz)
%continuous2NIFTI      Write to disk the continuous DBF solution in nifti format.
%
%   continuous2NIFTI(fNDiff,fPDD,fAmountDiff,mask,path,name,DPDD,DTra,spx,spy,spz) 
%   Write to disk PATH with generic file names NAME, the continuos solution of DBF model.
%   All tensors have diffusion along the fiber DPDD and transversal DTra. The continuos 
%   solution parameters FNDIFF,FPDD,FAMOUNTDIFF are computed by function giveContinousPDDS.
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$

[ny,nx,nz] = size(fNDiff);

amount = zeros(ny,nx,nz,3);
tensors1 = zeros(ny,nx,nz,1,6);
PDDs1 = zeros(ny,nx,nz,3);
tensors2 = zeros(ny,nx,nz,1,6);
PDDs2 = zeros(ny,nx,nz,3);
tensors3 = zeros(ny,nx,nz,1,6);
PDDs3 = zeros(ny,nx,nz,3);


T = [DPDD 0 0 ; 0 DTra 0; 0 0 DTra];

fprintf('\nWriting DBF-NII continuous solution..\n');

for pz=1:nz
    fprintf('\nslice %d',pz);    
    for px=1:nx
        for py=1:ny
            if mask(py,px,pz)==0
                continue;
            end
            
            nAmount = squeeze(fAmountDiff(py,px,pz,:));
            [v,idx] = max(nAmount);
            
            direc = squeeze(fPDD(py,px,pz,idx,:));
            
            mtx = fromToRotation([1 ;0 ;0], direc);
            Ti = mtx * T * mtx';
            
            Ti = Ti * fAmountDiff(py,px,pz,1); % for Gary's visor
            
            tensors1(py,px,pz,1,1)= Ti(1,1);
            tensors1(py,px,pz,1,2)= Ti(2,1);
            tensors1(py,px,pz,1,3)= Ti(2,2);
            tensors1(py,px,pz,1,4)= Ti(3,1);
            tensors1(py,px,pz,1,5)= Ti(3,2);
            tensors1(py,px,pz,1,6)= Ti(3,3);
            
            PDDs1(py,px,pz,:) = fPDD(py,px,pz,idx,:);
            amount(py,px,pz,1) = fAmountDiff(py,px,pz,idx);
            
            if fNDiff(py,px,pz)>1 
                nAmount(idx) = 0;
                [v,idx2] = max(nAmount);
                
                direc = squeeze(fPDD(py,px,pz,idx2,:));
                
                
                mtx = fromToRotation([1 ;0 ;0], direc);
                Ti = mtx * T * mtx';
                Ti = Ti * fAmountDiff(py,px,pz,idx2); % for Gary's visor
                
                tensors2(py,px,pz,1,1)= Ti(1,1);
                tensors2(py,px,pz,1,2)= Ti(2,1);
                tensors2(py,px,pz,1,3)= Ti(2,2);
                tensors2(py,px,pz,1,4)= Ti(3,1);
                tensors2(py,px,pz,1,5)= Ti(3,2);
                tensors2(py,px,pz,1,6)= Ti(3,3);

                PDDs2(py,px,pz,:) = fPDD(py,px,pz,idx2,:);
                amount(py,px,pz,2) = fAmountDiff(py,px,pz,idx2);
            end
            
            if fNDiff(py,px,pz)>2
                nAmount(idx2) = 0;
                [v,idx3] = max(nAmount);
                
                direc = squeeze(fPDD(py,px,pz,idx3,:));
                
                
                mtx = fromToRotation([1 ;0 ;0], direc);
                Ti = mtx * T * mtx';
                Ti = Ti * fAmountDiff(py,px,pz,idx3); % for Gary's visor
                
                tensors3(py,px,pz,1,1)= Ti(1,1);
                tensors3(py,px,pz,1,2)= Ti(2,1);
                tensors3(py,px,pz,1,3)= Ti(2,2);
                tensors3(py,px,pz,1,4)= Ti(3,1);
                tensors3(py,px,pz,1,5)= Ti(3,2);
                tensors3(py,px,pz,1,6)= Ti(3,3);

                PDDs3(py,px,pz,:) = fPDD(py,px,pz,idx3,:);
                amount(py,px,pz,3) = fAmountDiff(py,px,pz,idx3);
            end
                        
        end
    end
end



saveMatlab2Nifti([path  'sizeCompartment_' name '.nii'],amount,'NONE',spx,spy,spz);
saveMatlab2Nifti([path  'tensors1_' name '.nii'],tensors1,'MATRIX_SYM',spx,spy,spz);
saveMatlab2Nifti([path  'tensors2_' name '.nii'],tensors2,'MATRIX_SYM',spx,spy,spz);
saveMatlab2Nifti([path  'tensors3_' name '.nii'],tensors3,'MATRIX_SYM',spx,spy,spz);


saveMatlab2Nifti([path  'nTens_' name '.nii'],fNDiff,'NONE',spx,spy,spz);


fPDD(:,:,:,1,:) = PDDs1; %Put in sorted way
fPDD(:,:,:,2,:) = PDDs2;
fPDD(:,:,:,3,:) = PDDs3;

saveMatlab2Nifti([path  'fPDD_' name '.nii'],fPDD,'NONE',spx,spy,spz);


