function alphas= fitDBF_NNLS(s0,DW,mask,DBFs)
%fitDBF_NNLS      Fit the DBF model to a volume of images by NNLS method.
%
%   alphas= fitDBF_NNLS(s0,DW,mask,DBFs) 
%   Given 4D DW images volume, the 3D volume S0, the activity MASK and the DBFs model matrix,
%   it computes the 4D volume of ALPHAS coefficients associated to the DBF solution.
%   The results in ALPHAS is a field of vectors where each entry is associated to a discrete 
%   direction as explained in:
%   A. Ramirez-Manzanares etal, Diffusion Basis Functions Decomposition for Estimating 
%   White Matter Intravoxel Fiber Geometry. IEEE Trans. Med. Imaging 26(8): 2007.%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$
 
[m n] = size(DBFs);
[ny,nx,nz,m] = size(DW);

alphas = zeros(ny,nx,nz,n);

fprintf('\nNNLS DBF Fitting. Now Processing:');    

for pz=1:nz
    fprintf('\nslice %d',pz);    
    for px=1:nx
        for py=1:ny
            if mask(py,px,pz)==0
                continue;
            end
            [Si,S0r] = DWMRI_selectPosVol(DW, s0, py,px,pz);
            A = S0r*DBFs;                       
            alphas(py,px,pz,:)  = lsqnonneg(A,Si); 
        end
    end
end

