function LPS = linearPlanarSphericalCoeffs(lambdas)
%analizeLambdasHighLinearlyAnisotropic      Computes Westin's linear, planar, and spherical coefficients.
%
%   LPS = analizeLambdasHighLinearlyAnisotropic(lambdas) 
%   given an 4D image LAMBDAS with the 3 eigen values per voxel (sorted, such that LAMBDAS(:,:,:,3) is the 
%   biggest eigenvalue), 4D LAPS image contains the  Westin's (1999) linear, planar, and spherical coefficients in the volumes
%   LAPS(:,:,:,1), LAPS(:,:,:,2), and LAPS(:,:,:,3) respectively.
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$


sumL = lambdas(:,:,:,1) + lambdas(:,:,:,2) + lambdas(:,:,:,3);
LAni =    (lambdas(:,:,:,3) - lambdas(:,:,:,2)) ./ (sumL);
LPla = (2*(lambdas(:,:,:,2) - lambdas(:,:,:,1))) ./ (sumL);
LSph = (3*lambdas(:,:,:,1)) ./ (sumL);


LAni(sumL == 0) = 0;
LPla(sumL == 0) = 0;
LSph(sumL == 0) = 0;

LPS = zeros(size(lambdas));
LPS(:,:,:,1) = LAni;
LPS(:,:,:,2) = LPla;
LPS(:,:,:,3) = LSph;


